//  IoT電子工作キット sLab-Remo2
//  2-1-5.赤外線リモコン受信

const byte IR_R_PIN = 4;     // ①GPI4でリモコン受信

// 起動時の初期設定
void setup() {
  Serial.begin(115200);
  Serial.println();
  Serial.println("IrRecvStart");
  pinMode(IR_R_PIN, INPUT);
}
// 赤外線受信処理の繰り返し
void loop() {
  if ( irRecv () ) {          // ②赤外線受信処理の実行
    Serial.println();
    Serial.println("RcvOK");  // ③信号を正常に受信した場合に表示
  } else {
    Serial.println();
    Serial.println("NoSig");  // ④30秒間信号がない場合に表示
  }
}

// ⑤赤外線受信（信号受信 or 15秒間を処理）
bool irRecv () {
  // ⑥irRecv関数内で利用する変数（ローカル変数）を定義
  unsigned short irCount = 0;   // HIGH,LOWの信号数
  unsigned long lastt = 0;      // 1つ前の経過時間を保持
  unsigned long deltt = 0;      // 1つ前の経過時間を保持
  unsigned long sMilli;         // 本処理の開始時間
  unsigned long sMicro;         // 処理の開始時間
  unsigned long wMicro;         // 待ち開始時間
  bool rState = 0;              // 赤外線受信モジュールの状態 0:LOW,1:HIGH
  sMilli = millis();            // ⑤現在のシステム時間を取得（ミリ秒で取得）
  // ⑦特定条件(信号受信 or 15秒経過)するまで無限ループ
  while(1) {
    // ⑧Ir受信を待つ開始時間を取得
    wMicro = micros();          // 現在のシステム時間を取得（マイクロ秒で取得）
    // ⑨反転信号の受信待ち
    while (digitalRead(IR_R_PIN) == rState) {
      // ⑩待ち始めて0.5秒以上経過したら
      if (micros() - wMicro > 500000) {
        // ⑪待ち始めて0.5秒以上経過したら
        if ( irCount > 10 ) {
          return true;    // ⑫正常に完了
        }
        // ⑬0,1信号が10個以上ない場合は雑音のため再度ゼロから受信
        irCount = 0;
        delay(1); // watchdogタイマ用（３秒以内にリセットさせる必要あり）
      }
      // ⑭処理が15秒以上経過したらT.O.
      if ( millis() - sMilli > 15000 ) {
        return false;    // ⑮15秒経過で終了（受信なし）
      }
    }
    // ⑯信号受信開始時の現在の時間や経過時間を取得
    if ( irCount == 0 ) {
      sMicro = micros();
      lastt = 0;
      irCount++;
      Serial.println("ir:");
    // ⑰信号受信処理開始後の処理（irCountが1以上）
    } else {
      // ⑱赤外線受信部の状態変化が最後に変化した時間からの経過時間を計算
      deltt = ( (micros() - sMicro)/ 10 ) - lastt;
      // ⑲次回経過時間計算のため最後に変化した経過時間を保存
      lastt = lastt + deltt;
      irCount++;
      Serial.print(deltt);
      Serial.print(",");
    }
    // ⑳次回While内で状態変化を検知する値を変更
    rState = !rState;
  }
}
