//  IoT電子工作キット sLab-Remo2
//  3-3.スマホでLEDを制御

// ①ライブラリを読み込み
#include <WiFi.h>
#include <ESPAsyncWebServer.h>

// ②ピン配置
const byte LED_PIN = 12;  // LED緑
bool ledFlag = true;      // LED制御フラグ

// ③Wi-Fi設定
const char *ssid      = "##### SSID #####";
const char *password  = "### PASSWORD ###";
IPAddress ip(192, 168, 1, 123);     // IPアドレス（本機が利用するIP）
IPAddress gateway(192, 168, 1, 1);  // デフォルトゲートウェイ
IPAddress subnet(255, 255, 255, 0); // サブネットマスク

// ④WebServerを定義（ポート80番を使用）
AsyncWebServer webServer ( 80 );

void setup(void) {
  // ⑤Serial設定
  Serial.begin(115200);
  Serial.println ( );
  // ⑥LED設定
  pinMode ( LED_PIN, OUTPUT );
  // ⑦無線Wi-Fi接続
  WiFi.config( ip, gateway, subnet );
  WiFi.begin ( ssid, password );

  // ⑧Wi-Fi接続処理（接続するまで無限ループ）
  while ( WiFi.status() != WL_CONNECTED ) {
    // ⑨LEDを１秒毎に点滅する
    ledFlag = !ledFlag;
    digitalWrite(LED_PIN, ledFlag);
    // ⑩１秒間Wait
    delay ( 1000 );
    Serial.print ( "." );
  }
  // ⑪Wi-Fi接続できたのでシリアルモニターにIPアドレス表示
  Serial.print ( "Wi-Fi Connected! IP address: " );
  Serial.println ( WiFi.localIP() );
  // ⑫LED点灯（Wi-Fi接続状態）
  digitalWrite ( LED_PIN, true );

  // ⑬WebServerの受信処理"/"を設定
  webServer.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    sendHtml(request);          // ⑭Webページ内容を送信
  });
  // ⑮WebServerの受信処理"/on"を設定
  webServer.on("/on", HTTP_GET, [](AsyncWebServerRequest *request){
    digitalWrite(LED_PIN, HIGH);  // ⑯LEDピンをHIGHにしてLED点灯
    sendHtml(request);          // ⑰Webページ内容を送信
  });
  // ⑱WebServerの受信処理"/off"を設定
  webServer.on("/off", HTTP_GET, [](AsyncWebServerRequest *request){
    digitalWrite(LED_PIN, LOW);   // ⑲LEDピンをHIGHにしてLED点灯
    sendHtml(request);          // ⑳Webページ内容を送信
  });
  // ㉑WebServer起動処理
  webServer.begin();
  Serial.println ( "Web server started" );
}

void loop(void){
}

// ㉒Webページ内容(HTML)を送信
void sendHtml(AsyncWebServerRequest *request) {
  String res_html =             // ㉓返答するHTML(文字列)を設定
  "<center>\
    <h2>LED Control</h2>\
    <p><button type='button' onclick='location.href=\"/on\"' \
      style='width:250px;height:60px;'>LED-ON</button></p>\
    <p><button type='button' onclick='location.href=\"/off\"' \
      style='width:250px;height:60px;'>LED-OFF</button></p>\
  </center>";
  // ㉔HTMLを返答する
  request->send ( 200, "text/html", res_html );
  Serial.println ( "send html" );
}
