//  IoT電子工作キット sLab-Remo2
//  4-2-2.SPIFFSによるファイル操作2

// ①ライブラリを読み込み
#include <WiFi.h>
#include <ESPAsyncWebServer.h>
#include <SPIFFS.h>

// ②ピン配置
const byte LED_PIN = 12;  // LED緑
bool ledFlag = true;      // LED制御フラグ

// ③Wi-Fi設定
const char *ssid      = "##### SSID #####";
const char *password  = "### PASSWORD ###";
IPAddress ip(192, 168, 1, 123);     // IPアドレス（本機が利用するIP）
IPAddress gateway(192, 168, 1, 1);  // デフォルトゲートウェイ
IPAddress subnet(255, 255, 255, 0); // サブネットマスク

// ④WebServerを定義
AsyncWebServer webServer ( 80 );

void setup(void) {
  
  // ⑤Serial設定
  Serial.begin(115200);
  Serial.println ( );
  // ⑥SPIFFS開始
  SPIFFS.begin();
  // ⑦LED設定
  pinMode ( LED_PIN, OUTPUT );
  // ⑧無線Wi-Fi接続
  WiFi.config( ip, gateway, subnet );
  WiFi.begin ( ssid, password );

  // ⑨Wi-Fi接続処理（接続するまで無限ループ）
  while ( WiFi.status() != WL_CONNECTED ) {
    // ⑩LEDを１秒毎に点滅する
    ledFlag = !ledFlag;
    digitalWrite(LED_PIN, ledFlag);
    // ⑪１秒間Wait
    delay ( 1000 );
    Serial.print ( "." );
  }
  // ⑫Wi-Fi接続できたのでシリアルモニターにIPアドレス表示
  Serial.print ( "Wi-Fi Connected! IP address: " );
  Serial.println ( WiFi.localIP() );
  // ⑬LED点灯（Wi-Fi接続状態）
  digitalWrite ( LED_PIN, true );

  // ⑭WebServerの受信処理"/"を設定
  webServer.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    sendHtml(request);    // ⑮Webページ内容を送信
  });
  // ⑯WebServer起動処理
  webServer.begin();
  Serial.println ( "Web server started" );
}

void loop(void){
}

// ⑰HTMLファイルを送信する関数
void sendHtml(AsyncWebServerRequest *request) {
  // ⑱HTMLファイル"hello.html"を送信する
  request->send(SPIFFS, "/hello.html", "text/html");
  Serial.println ( "Send Html" );
}
