//  IoT電子工作キット
//  4-3.スマホで操作する家電リモコン（屋内）

// ①ライブラリを読み込み
#include <WiFi.h>
#include <EEPROM.h>
#include <DHT.h>
#include <SPIFFS.h>
#include <ESPAsyncWebServer.h>

// ②ピン配置
const byte LED_PIN  = 12;  // LED緑
const byte IR_R_PIN =  4;  // リモコン受信
const byte IR_S_PIN = 32;  // リモコン送信
bool ledFlag = true;       // LED制御フラグ

// ③EEPROMで利用する型（構造体）を宣言（ボタン名を保存する型）
struct st_remocon {
  char remo_name[65];
};

// ④Wi-Fi設定
const char *ssid      = "##### SSID #####";
const char *password  = "### PASSWORD ###";
IPAddress ip(192, 168, 1, 123);     // IPアドレス（本機が利用するIP）
IPAddress gateway(192, 168, 1, 1);  // デフォルトゲートウェイ
IPAddress subnet(255, 255, 255, 0); // サブネットマスク

// ⑤WebServerを定義 ポート番号:80
AsyncWebServer webServer ( 80 );
// ⑥リモコンデータ保存用 (unsigned short型の配列1500個)
unsigned short irData[1500];

void setup(void) {
  // ⑦Serial設定
  Serial.begin(115200);
  Serial.println ( );
  // ⑧SPIFFS開始
  SPIFFS.begin();
  // ⑨EEPROM開始（サイズ指定：ボタン名10個分[65*10]）
  EEPROM.begin(650);
  // ⑩ピンモード設定
  pinMode ( LED_PIN,  OUTPUT );
  pinMode ( IR_R_PIN, INPUT );
  pinMode ( IR_S_PIN, OUTPUT );
  // ⑪無線Wi-Fi接続
  WiFi.config( ip, gateway, subnet );
  WiFi.begin ( ssid, password );

  // ⑫Wi-Fi接続処理（接続するまで無限ループ）
  while ( WiFi.status() != WL_CONNECTED ) {
    // ⑬LEDを１秒毎に点滅する
    ledFlag = !ledFlag;
    digitalWrite(LED_PIN, ledFlag);
    delay ( 1000 );
    Serial.print ( "." );
  }
  // ⑭Wi-Fi接続できたのでシリアルモニターにIPアドレス表示
  Serial.print ( "Wi-Fi Connected! IP address: " );
  Serial.println ( WiFi.localIP() );
  // ⑮LED点灯（Wi-Fi接続状態）
  digitalWrite ( LED_PIN, true );

  // ⑯WebServer処理設定（制御画面を送信）
  webServer.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "Top" );
      request->send(SPIFFS, "/top.html", "text/html");
  });
  // ⑰WebServer処理設定（制画面のJavascriptを送信）
  webServer.on("/rem.js", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "js" );
      request->send(SPIFFS, "/rem.js", "text/html");
  });
  // ⑱WebServer処理設定（設定画面を送信）
  webServer.on("/set", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "set" );
      request->send(SPIFFS, "/set.html", "text/html");
  });
  // ⑲WebServer処理設定（ボタン情報送信）
  webServer.on("/getrem", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "getrem" );
      getRemocon(request);
  });
  // ⑳WebServer処理設定（リモコン受信設定）
  webServer.on("/setrem", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "setrem" );
      setRemocon(request);
  });
  // ㉑WebServer処理設定（リモコン送信）
  webServer.on("/cntrem", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "cntrem" );
      contRemocon(request);
  });
  // ㉒WebServer開始処理
  webServer.begin();
  Serial.println ( "Web server started" );
}

// ㉓loop処理
void loop(void){
}

// ㉔ボタン情報送信関数
void getRemocon(AsyncWebServerRequest *request) {
  // ㉕送信データを作成(JSON形式)
  String senddata = "{";
  // ㉖EEPROMデータを保存する変数を宣言
  st_remocon remRom;
  // ㉗ボタン情報10個分を読み出し返答する
  for (byte i = 0; i < 10; i++) {
    // ㉘EEPROMメモリ位置を計算
    int memPos = (65 * i);
    // ㉙前回の値'O:'が残らないように消す
    remRom.remo_name[0] = 'n';
    // ㉚EEPROMからデータ取得
    EEPROM.get<st_remocon>(memPos, remRom);
    // ㉛データが保存されているかCheck
    if (remRom.remo_name[0] == 'O' && remRom.remo_name[1] == ':' ) {
      // ㉜返答文字列長が１を超える場合は","をつける（２つ目以降との区切り）
      if (senddata.length() > 1) {
        senddata += ",";
      }
      // ㉝返答する値を一度String型に置き換える（"O:"を削除するため）
      String getirname = String(remRom.remo_name);
      // ㉞返答文字列作成（"O:"を削除するため2から最後まで）
      senddata += "\"" + (String)i + "\":\"" + getirname.substring(2,getirname.length()) + "\"";
    }
  }
  // ㉟最後にJSONデータを閉じるため"}"をつける
  senddata += "}";
  // ㊱作成した返答（JSON）データをWebサーバから返答する
  request->send(200, "text", senddata);
  Serial.println( "getRemocon:" + senddata);
}
