// ①赤外線受信（信号受信 or 30秒間を処理）
void setRemocon (AsyncWebServerRequest *request) {
  // ②利用する変数を宣言
  unsigned short irCount = 0;   // HIGH,LOWの信号数
  uint32_t lastt = 0;           // 1つ前の経過時間を保持
  uint32_t deltt = 0;           // 1つ前の経過時間を保持
  uint32_t sMilli;              // 本処理の開始時間
  uint32_t wMilli;              // 赤外線の待ち開始時間
  uint32_t sMicro;              // 処理の開始時間
  uint32_t cMicro;              // 現在時間
  bool rState = 0;              // 赤外線受信モジュールの状態 0:LOW,1:HIGH
  // ③現在の時間をミリ秒で取得
  sMilli = millis();
  // ④特定条件(信号受信 or 30秒経過)するまで無限ループ
  while(1) {
    // ⑤Ir受信を待つ開始時間を取得
    wMilli = millis();
    // ⑥反転信号の受信待ち
    while (digitalRead(IR_R_PIN) == rState) {
      // ⑦待ち始めて0.5秒以上経過したら
      if (millis() - wMilli > 500) {
        // ⑧待ち始めて0.5秒以上経過したら
        if ( irCount > 10 ) {
          Serial.println("");
          Serial.println("recvOK!");
          delay(1);
          // ⑨ボタン名をEEPROMへ、リモコンデータをファイルへ保存
          //xTaskCreatePinnedToCore(saveIr,"saveIr", 4096, ((irCount - 1), request), 1, NULL, 1); //Task1実行
          if (saveIr( (irCount - 1), request)) {
            // ⑩正常に終了したため、WebサーバからOKを返答
            request->send(200, "text", "OK");
            Serial.println("setRemocon OK");
          } else {
            // ⑪信号保存に失敗したためNGを返答
            request->send(200, "text", "NG");
            Serial.println("setRemocon NG");
          }
          return;    // ⑪正常に完了
        }
        // ⑫0,1信号が10個以上ない場合は雑音のため再度ゼロから受信
        irCount = 0;
      }
      // ⑬処理が15秒以上経過したらタイムアウト
      if ( millis() - sMilli > 15000 ) {
        request->send(200, "text", "NG");// ⑭タイムアウトしたのでNGを送信
        Serial.println("setRemocon T.O.");
        return;                          // ⑮15秒経過で処理終了（受信なし）
      }
    }
    // ⑯信号受信開始時の現在の時間や経過時間を取得
    if ( irCount == 0 ) {
      sMicro = micros();
      lastt = 0;
      irCount++;
      Serial.print("ir:");
    // ⑰信号受信処理開始後の処理（irCountが1以上）
    } else {
      // ⑱赤外線受信部の状態変化が最後に変化した時間からの経過時間を計算
      cMicro = micros();
      deltt = ( (cMicro - sMicro)/ 10 ) - lastt;
      irData[(irCount - 1 )] = deltt;
      // ⑲次回経過時間計算のため最後に変化した経過時間を保存
      lastt = lastt + deltt;
      irCount++;
      Serial.print( deltt );
      Serial.print(",");
    }
    // ⑳次回While内で状態変化を検知する値を変更
    rState = !rState;
  }
}

// ㉑ボタン名をEEPROMへ、リモコンデータをファイルへ保存
bool saveIr(unsigned short irLength, AsyncWebServerRequest *request){
  String setirname = "";
  String setNumStr = "";
  // ㉒ボタン番号の取得とチェック(HTTP GETリクエストのパラメータ)
  if (request->hasParam("n")) {
    setNumStr = request->getParam("n")->value();
  } else {
    return false;
  }
  // ㉓ボタン名の取得とチェック(HTTP GETリクエストのパラメータ)
  if (request->hasParam("a")) {
    setirname = request->getParam("a")->value();
  } else {
    return false;
  }
  // ㉔ボタン番号をString型をint型へ変換
  int setNum = setNumStr.toInt();
  // ㉕ボタン名に識別用文字"O:"を先頭に付与
  setirname = "O:" + setirname;
  // ㉖EEPROMに保存用の型が一致した変数を定義
  st_remocon remRom;
  // ㉗Stringからchar型に変換（終了文字を付与するためLengthは+1）
  setirname.toCharArray(remRom.remo_name, setirname.length()+1);
  // ㉘メモリ位置を計算しEEPROMへ書き込み
  int memPos = (65 * setNum);
  EEPROM.put<st_remocon>(memPos, remRom);
  EEPROM.commit();
  Serial.println("setIr:" + String(setNum) + ":" + setirname);
  // ㉙リモコン信号を保存するためファイル名を作成（ファイル名はボタン番号）
  String t_file = "/" + setNumStr;
  Serial.println( "recvFile:" + t_file);
  // ㉚ファイルを書き込みモードで開く
  File fw = SPIFFS.open(t_file.c_str(), "w");
  // ㉛リモコン信号の長さを最初に書き込み（最初の行）
  fw.println( String( irLength, HEX ) );
  // ㉜リモコン信号の0,1の時間長さを書き込み（２行目以降）
  for (int i = 0; i < irLength; i++) {
    fw.println( String( irData[i], HEX ) );
  }
  // ㉝書き込みが完了したのでファイルを閉じる
  fw.close();
  // ㉞正常に処理が完了したのでtrueを返す
  return true;
}

// ㉟赤外線送信処理
bool  contRemocon (String setNumStr) {  // 送信番号保存
  // ㊱変数宣言
  unsigned short irCount = 0; // HIGH,LOWの信号数
  unsigned long l_now = 0;    // 送信開始時間を保持
  unsigned long sndt = 0;     // 送信開始からの経過時間 
  // ㊲リモコン信号をファイルから取得（ファイル名はボタン番号）
  String t_file = "/" + setNumStr;
  Serial.println( "sendFile:" + t_file);
  // ㊳ファイルを開く
  File fr = SPIFFS.open(t_file.c_str(), "r");
  // ㊴ファイルを開く
  if(!fr || fr.isDirectory()){
    Serial.println("FileOpen NG");
    return false;
  }
  // ㊵最初の１行のみ読み出し信号長さ（0,1の数）を取得
  String snum = fr.readStringUntil('\n');
  // ㊶１行目の文字列を数値型に変換
  irCount = strtol(snum.c_str(), NULL, 16);
  Serial.print( "sendData:" + String(irCount));
  // ㊷一旦ファイルから変数irDataに読み出し保存
  for (int i = 0; i < irCount; i++) {
    snum = fr.readStringUntil('\n');
    irData[i] = strtol(snum.c_str(), NULL, 16);
  }
  fr.close();
  // ㊸送信開始時間を取得
  l_now = micros();
  // ㊹0,1の信号回数分をFor文でループ
  for (int i = 0; i < irCount; i++) {
    // ㊺送信開始からの信号終了時間を計算
    sndt += irData[i];
    do {
      // ㊻iが偶数なら赤外線ON、奇数ならOFFのまま
      // キャリア周波数38kHz(約26μSec周期の半分)でON時間で送信
      digitalWrite(IR_S_PIN, !(i&1));
      microWait(13);
      // ㊼キャリア周波数38kHz(約26μSec周期の半分)でOFF時間で送信
      digitalWrite(IR_S_PIN, 0);
      microWait(13);
    // ㊽送信開始からの信号終了時間が超えるまでループ
    } while (long(l_now + (sndt * 10) - micros()) > 0);
  }
  // ㊾正常に終了したので、trueを返答
  Serial.println(":End");
  return true;
}

// ㊿マイクロ秒単位で待つ
void microWait(signed long waitTime) {
  unsigned long waitStartMicros = micros();
  while (micros() - waitStartMicros < waitTime) {};
}
