// ①WebServer設定処理
void setup_webserver() {
  // ②WebServer処理設定（制御画面を送信）
  webServer.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "Top" );
      request->send(SPIFFS, "/top.html", "text/html");
  });
  webServer.on("/favicon.ico", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "favicon" );
      request->send(SPIFFS, "/favicon.ico", "image/x-icon");
  });
  // ③WebServerLED制御（LED制御）
  webServer.on("/led", HTTP_GET, [](AsyncWebServerRequest *request){
      String setLedVal = request->getParam("o")->value();
      if (setLedVal == "ON") {
        digitalWrite ( LED_PIN, true );
        Serial.println ( "led on" );
      } else {
        digitalWrite ( LED_PIN, false );
        Serial.println ( "led off" );
      }
      request->send(200, "text/plain", "OK");
  });
  // ④WebServer処理設定（制画面のJavascriptを送信）
  webServer.on("/rem.js", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "js" );
      request->send(SPIFFS, "/rem.js", "text/html");
  });
  // ⑤WebServer処理設定（設定画面を送信）
  webServer.on("/set", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "set" );
      request->send(SPIFFS, "/set.html", "text/html");
  });
  // ⑥WebServer処理設定（温度情報送信）
  webServer.on("/gettemp", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "gettemp" );
      getTemprature(request);
  });
  // ⑦WebServer処理設定（ボタン情報送信）
  webServer.on("/getrem", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "getrem" );
      getRemocon(request);
  });
  // ⑧WebServer処理設定（リモコン受信設定）
  webServer.on("/setrem", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "setrem" );
      setRemocon(request);
  });
  // ⑨WebServer処理設定（リモコン送信）
  webServer.on("/cntrem", HTTP_GET, [](AsyncWebServerRequest *request){
      Serial.println ( "cntrem" );
      // ⑩ボタン番号を取得とチェック(HTTP GETリクエストのパラメータ)及び送信処理
      if (request->hasParam("n") && contRemocon( request->getParam("n")->value() )) {
        request->send(200, "text", "OK");
        Serial.println("Web send OK");
      } else {
        request->send(200, "text", "NG");
        Serial.println("Web send NG");
      }
  });
  // ⑪WebServer開始処理
  DefaultHeaders::Instance().addHeader("Access-Control-Allow-Origin", "*");
  webServer.begin();
  Serial.println ( "Web server started" );
}

// ⑫ボタン情報送信関数
void getRemocon(AsyncWebServerRequest *request) {
  // ⑬送信データを作成(JSON形式)
  String senddata = "{";
  // ⑭EEPROMデータを保存する変数を宣言
  st_remocon remRom;
  // ⑮ボタン情報10個分を読み出し返答する
  for (byte i = 0; i < 10; i++) {
    // ⑯EEPROMメモリ位置を計算
    int memPos = (65 * i);
    // ⑰前回の値'O:'が残らないように消す
    remRom.remo_name[0] = 'n';
    // ⑱EEPROMからデータ取得
    EEPROM.get<st_remocon>(memPos, remRom);
    // ⑲データが保存されているかCheck
    if (remRom.remo_name[0] == 'O' && remRom.remo_name[1] == ':' ) {
      // ⑳返答文字列長が１を超える場合は","をつける（２つ目以降との区切り）
      if (senddata.length() > 1) {
        senddata += ",";
      }
      // ㉑返答する値を一度String型に置き換える（"O:"を削除するため）
      String getirname = String(remRom.remo_name);
      // ㉒返答文字列作成（"O:"を削除するため2から最後まで）
      senddata += "\"" + (String)i + "\":\"" + getirname.substring(2,getirname.length()) + "\"";
    }
  }
  // ㉓最後にJSONデータを閉じるため"}"をつける
  senddata += "}";
  // ㉔作成した返答（JSON）データをWebサーバから返答する
  request->send(200, "text", senddata);
  Serial.println( "getRemocon:" + senddata);
}

// ㉕温・湿・照度情報送信関数
void getTemprature(AsyncWebServerRequest *request) {
  // ㉖温湿度センサーの宣言
  DHT dht(TEMP_SNR, DHT11);
  // ㉗温湿度センサーの情報取得
  float g_humidity = dht.readHumidity();
  float g_temperature = dht.readTemperature();
  // ㉘温湿度センサーの情報表示
  Serial.print( "get_temp:" );
  Serial.print( String(g_temperature) );
  Serial.print( ":" );
  Serial.print( String(g_humidity) );
  // ㉙照度センサの情報取得（1〜100）
  unsigned short g_light = analogRead(LIGH_SNR) * 100 / 4095 / 0.8;
  // ㉚照度センサーの情報表示
  Serial.print( ":" );
  Serial.println( String(g_light) );
  // ㉛センサーデータを作成し送信
  String senddata = "{\"t\":" + String(g_temperature) + ",\"h\":" + String(g_humidity) + ",\"l\":" + String(g_light) + "}";
  request->send(200, "text", senddata);
  // ㉜送信したセンサーデータを表示
  Serial.print("gTem:");
  Serial.println(senddata);
}

// ㉝赤外線送信処理
void iftttMail () {
  // ㉞URL決定 maker.ifttt.com/trigger/{event}/with/key/{key}
  String value1 = "Motion is detected!";
  String pathUrl = "/trigger/" + String(iftttEvent) + "/with/key/" + String(iftttKey);
  String json = "{\"value1\":\"" + value1 + "\"}";
  delay(10);
  // ㉟LED消灯（処理中が判別できるように消灯している）
  digitalWrite(LED_PIN, false);// 処理中はLED消灯
  // ㊱HTTPS接続と結果判定
  if (!wifiClientHttps.connect(iftttHost, 443)) {
    // ㊲HTTPS接続失敗時のシリアルモニタ表示
    Serial.println("Connection failed!");
  } else {
    // ㊳HTTPS接続成功時のシリアルモニタ表示
    Serial.println("Connected to ifttt!\n");
    // ㊴HTTPSデータ送信処理
    wifiClientHttps.println("POST https://" + String( iftttHost ) + pathUrl + " HTTP/1.1");
    wifiClientHttps.println("Host: " + String(iftttHost) + ":443");
    wifiClientHttps.println("Content-Type: application/json");
    wifiClientHttps.println("Connection: Keep-Alive");
    wifiClientHttps.println("Content-Length: "+ String(json.length()));
    wifiClientHttps.println("");
    wifiClientHttps.println(json);
    // ㊵HTTPS接続確認
    if (wifiClientHttps.connected()) {
      // ㊶ヘッダの受信表示処理
      Serial.println("--- Header ---");
      while (1) {
        String line = wifiClientHttps.readStringUntil('\n');
        Serial.println(line);
        if (line == "\r") {
          break; // ヘッダの末尾は\r\nのため、\r時に終了
        }
      }
      // ㊷本文(Body)の受信表示処理
      Serial.println("--- Body ---");
      while (wifiClientHttps.available()) {
        char c = wifiClientHttps.read();
        Serial.write(c);
      }
    }
    // ㊸HTTPS通信の終了
    wifiClientHttps.stop();
  }
  // ㊹終了処理（LED点灯、シリアルモニタ表示）
  digitalWrite(LED_PIN, true);
}
